#include <avr/interrupt.h>
#include <util/delay.h>
#define c    261
#define d    294
#define e    329
#define f    349
#define g    391
#define gS   415
#define a    440
#define aS   455
#define b    466
#define cH   523
#define cSH  554
#define dH   587
#define dSH  622
#define eH   659
#define fH   698
#define fSH  740
#define gH   784
#define gSH  830
#define aH   880
 
uint16_t notes[] = {a, a, a, f, cH, a, f, cH, a, eH, eH, eH, fH, cH, gS, f, cH, a, aH, a, a, aH, gSH, gH, fSH, fH, fSH, aS, dSH, dH, cSH, cH, b, cH, f, gS, f, a, cH, a, cH, eH, aH, a, a, aH, gSH, gH, fSH, fH, fSH, aS, dSH, dH, cSH, cH, b, cH, f, gS, f, cH, a, f, cH, a};
 
uint16_t durations[] = {500, 500, 500, 350, 150, 500, 350, 150, 650, 500, 500, 500, 350, 150, 500, 350, 150, 650, 500, 300, 150, 400, 200, 200, 125, 125, 250, 250, 400, 200, 200, 125, 125, 250, 125, 500, 375, 125, 500, 375, 125, 650, 500, 300, 150, 400, 200, 200, 125, 125, 250, 250, 400, 200, 200, 125, 125, 250, 250, 500, 375, 125, 500, 375, 125, 650};
 
uint8_t num_notes = sizeof(notes) / sizeof(notes[0]);

//Functie ce itereaza prin becuri
void nextLed(int index) {
//Daca este ultimul bec o luam de la capat
        if (index == 16) index = 0;
        if (index < 8) {
                PORTA = 0;
                index++;
                if(index == 8) {
                        PORTC = 1;
                }
                else {
                        PORTA ^= 1 << index;
                }
        }
        else {
                index -=8;
                PORTC = 0 ;
                index++;
                if(index == 8) {
                        PORTA = 1;
                }
                else {
                        PORTC ^= 1 << index;
                }
        }
}

//Functie ce da togle la un nivel(deschide tranzistorul)
void togleLevel(int index) {
        PORTB ^= 1 << index;
}

//Functie luata din laborator
void timer1_init()
{
        //timer1 este configurat cu modul FastPWM, top ICR1 si prescaler 1024
        TCCR1A |= (2 << WGM10);
        TCCR1B |= (5 << CS10) | (3 << WGM12);
 
        TCCR1A |= (2 << COM1B0) | (2 << COM1A0);
 
        //modul curent numara pana la ICR1
        ICR1 = 60000;
        //duty cycle-ul este setat initial la 25%
        OCR1B = ICR1 / 4;
        //duty cycle-ul initial pentru speaker (OC1A) este 50%
        OCR1A = ICR1 / 2;
}
 
int main(void)
{
        int i;
		//Ajustam frecventa si durata pentru difuzor
                for(i = 0; i < num_notes; ++i) {
                        notes[i] -= (notes[i] / 50);
                        durations[i] -=(durations[i] / 20);
                }
		//Pini de iesire
        DDRD |= (1 << PD5);
        DDRA |= 0xFF;
        DDRC |= 0xFF;
        //tranzistorii care controleaza fiecare plan
        DDRB |= 0xFF;
 
        //initializam PWM-ul
        timer1_init();
 
        uint8_t note_remaining = 1;
        int8_t note_position = -1;
 
        //Stingem toate ledurile
        PORTA = 0;
        PORTC = 0;
 
        //Aprindem primul plan
        PORTB = 1;
        int iter = 0;
        int led= -2;
        int level = 0;
        while(1)
        {
                if(note_remaining == 0)
                {
				//Daca schimbam nota
                        if(++led == 15) {
						//Daca am terminat cu un nivel
                                togleLevel(level++);
                                if(level == 4) level = 0;
                                togleLevel(level);
                                led = -1;
                                PORTA = 0;
                                PORTC = 0;
                        }
						nextLed(led);
                        note_position = (note_position + 1) % num_notes;
                        note_remaining = durations[note_position];
                        ICR1 = 16000000 / 1024 / notes[note_position];
                        OCR1A = ICR1 / 2;
                        TCNT1 = 0;
                }
                note_remaining--;
			_delay_ms(25);
        }
        return 0;
}